import javax.swing.*;
import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.util.function.Consumer;

public class FormWarshipConfig extends JFrame {
    private final Action<DrawingWarship> event = new Action<>();
    private DrawingWarship warship;
    BlockCount block;
    private IDrawingObjectBlock _block=null;
    private JPanel MainPanel;
    private JPanel GroupBoxConfig;
    private JSpinner SpinnerSpeed;
    private JSpinner SpinnerWeight;
    private JCheckBox CheckBoxAntenna;
    private JCheckBox CheckBoxHelipad;
    private JCheckBox CheckBoxMissile;
    private JLabel SpeedLabel;
    private JLabel WeightLabel;
    private JPanel ColorPanel;
    private JPanel RedPanel;
    private JPanel OrangePanel;
    private JPanel GreenPanel;
    private JPanel BluePanel;
    private JPanel YellowPanel;
    private JPanel CianPanel;
    private JPanel PinkPanel;
    private JPanel GrayPanel;
    private JLabel CountBlockLabel;
    private JSpinner SpinnerBlock;
    private JPanel TypeBlock;
    private JLabel SquareBlockLabel;
    private JLabel TriangleBlockLabel;
    private JLabel RoundBlockLabel;
    private JPanel PreviewPanel;
    private JLabel BodyColorLabel;
    private JLabel DopColorLabel;
    private JLabel BaseWarshipLabel;
    private JLabel AdvancedWarshipLabel;
    private JButton ButtonAdd;
    private JButton ButtonCancel;
    private JPanel PictureBox;

    public FormWarshipConfig(){
        this.setTitle("Создание объекта");
        this.setSize(900,500);
        this.setContentPane(MainPanel);
        this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);

        SquareBlockLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        TriangleBlockLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        RoundBlockLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        BodyColorLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        DopColorLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        BaseWarshipLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        AdvancedWarshipLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));

        SpinnerBlock.setModel(new SpinnerNumberModel(2, 2, 6, 2));
        SpinnerSpeed.setModel(new SpinnerNumberModel(100, 100, 1000, 10));
        SpinnerWeight.setModel(new SpinnerNumberModel(10000, 10000, 20000, 100));

        var dragAdapter = new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                super.mouseReleased(e);
                setCursor(new Cursor(Cursor.HAND_CURSOR));
            }

            @Override
            public void mouseReleased(MouseEvent e) {
                super.mouseReleased(e);
                dispatchDrop((JComponent) e.getSource());
            }
        };

        RedPanel.addMouseListener(dragAdapter);
        GreenPanel.addMouseListener(dragAdapter);
        BluePanel.addMouseListener(dragAdapter);
        YellowPanel.addMouseListener(dragAdapter);
        OrangePanel.addMouseListener(dragAdapter);
        GrayPanel.addMouseListener(dragAdapter);
        CianPanel.addMouseListener(dragAdapter);
        PinkPanel.addMouseListener(dragAdapter);

        BaseWarshipLabel.addMouseListener(dragAdapter);
        AdvancedWarshipLabel.addMouseListener(dragAdapter);
        SquareBlockLabel.addMouseListener(dragAdapter);
        TriangleBlockLabel.addMouseListener(dragAdapter);
        RoundBlockLabel.addMouseListener(dragAdapter);

        ButtonAdd.addActionListener(e -> {
            event.Invoke(warship);
            dispose();
        });
        ButtonCancel.addActionListener(e -> dispose());
    }

    public void addListener(Consumer<DrawingWarship> listener) {
        event.addListener(listener);
    }

    public void dispatchDrop(JComponent droppedComponent) {
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        if (droppedComponent == null) {
            return;
        }

        if (droppedComponent instanceof JPanel panel) {
            if (BodyColorLabel.getMousePosition() != null) {
                warship.SetBodyColor(panel.getBackground());
            }
            if (DopColorLabel.getMousePosition() != null && warship instanceof DrawingAdvancedWarship advanced) {
                advanced.SetDopColor(panel.getBackground());
            }
        }
        if (droppedComponent instanceof JLabel label && PictureBox.getMousePosition() != null) {
            int speed = (Integer) SpinnerSpeed.getValue();
            int weight = (Integer) SpinnerWeight.getValue();
            int countBlock = (Integer) SpinnerBlock.getValue();
            boolean antenna = CheckBoxAntenna.isSelected();
            boolean helipad = CheckBoxHelipad.isSelected();
            boolean missile = CheckBoxMissile.isSelected();

            if (label == BaseWarshipLabel) {
                warship = new DrawingWarship(speed, weight, countBlock,Color.WHITE);
            } else if (label == AdvancedWarshipLabel) {
                warship = new DrawingAdvancedWarship(speed, weight,countBlock, Color.WHITE, Color.WHITE, helipad, antenna,missile);
            } else if (warship != null && label == SquareBlockLabel) {
                warship.SetBlocks(new DrawingBlock(countBlock));
            } else if (warship != null && label == TriangleBlockLabel) {
                warship.SetBlocks(new DrawingTriangleBlocks(countBlock));
            } else if (warship != null && label == RoundBlockLabel) {
                warship.SetBlocks(new DrawingRoundBlocks(countBlock));
            }
        }
        repaint();
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        if (warship != null) {
            g = PictureBox.getGraphics();
            warship.SetPosition(10, 10, PictureBox.getWidth(), PictureBox.getHeight());
            warship.DrawTransport((Graphics2D) g);
        }
    }
}
